@@ -2,9 +2,11 @@ setClass("AllModelClass",
          representation(pars='list', 
                         model='list',
                         K='numeric', 
-                        G2='numeric', 
+                        G2='numeric',
+                        X2='numeric',
                         df='numeric', 
                         p='numeric', 
+                        p.X2='numeric',
                         AIC='numeric', 
                         AICc='numeric',
                         F='matrix', 
@@ -17,9 +19,11 @@ setClass("AllModelClass",
                         itemloc = 'numeric',                        
                         BIC='numeric', 
                         SABIC='numeric',
-                        RMSEA='numeric',                         
+                        RMSEA='numeric',
+                        RMSEA.X2='numeric',
                         null.mod = 'S4', 
                         TLI = 'numeric',                          
+                        TLI.X2 = 'numeric',
                         logLik='numeric',                                    
                         SElogLik='numeric',
                         Call='call',
@@ -37,6 +41,8 @@ setClass("AllModelClass",
                         method='character',
                         itemtype='character',
                         time='numeric',
+                        CFI='numeric',
+                        CFI.X2='numeric',
                         'VIRTUAL'),    
          validity = function(object) return(TRUE)
 )                       
