@@ -57,6 +57,19 @@ setClass("partcomp", contains = 'AllItemsClass')
 
 setClass("mcm", contains = 'AllItemsClass')
 
+setClass("nestlogit", contains = 'AllItemsClass',
+         representation = representation(correctcat='integer'))
+
+setClass('custom', contains = 'AllItemsClass', 
+         representation = representation(name='character',
+                                         P='function',
+                                         gr='function',
+                                         usegr='logical',
+                                         hss='function',
+                                         usehss='logical',
+                                         userdata='matrix',
+                                         useuserdata='logical'))    
+
 #--------------------------------------------------------------------------
 
 #Generics
@@ -79,4 +92,4 @@ setGeneric("itemplot.internal",  function(object, ...) standardGeneric("itemplot
 
 setGeneric("fscores.internal", function(object, ...) standardGeneric("fscores.internal"))
 
-setGeneric("summary", function(object, ...) standardGeneric("summary"))
\ No newline at end of file
+setGeneric("summary", function(object, ...) standardGeneric("summary"))
