@@ -3,16 +3,18 @@
 #' \code{multipleGroup} performes a full-information
 #' maximum-likelihood multiple group analysis for dichotomous and polytomous
 #' data under the item response theory paradigm using either Cai's (2010)
-#' Metropolis-Hastings Robbins-Monro algorithm or with an EM approach. 
+#' Metropolis-Hastings Robbins-Monro (MHRM) algorithm or with an EM approach. 
 #'  
 #' By default the estimation in \code{multipleGroup} assumes that the models are maximally 
 #' independent, and therefore could initially be performed by sub setting the data and running identical
 #' models with \code{confmirt} or \code{mirt} and aggregating the results (e.g., log-likelihood). 
-#' However, constrains may be imposed across groups by invoking various \code{invariance} keywords
-#' or by inputing user defined \code{freepars}, \code{constrain}, and \code{startvalues} lists.  
+#' However, constrains may be imposed across groups by invoking various \code{invariance} keywords and 
+#' \code{constrain = ...} arguments, and by inputing user specified design matrix from 
+#' \code{mod2values} or from passing \code{pars = 'values'}.  
 #' 
 #' @aliases multipleGroup coef,MultipleGroupClass-method summary,MultipleGroupClass-method
-#' anova,MultipleGroupClass-method 
+#' anova,MultipleGroupClass-method plot,MultipleGroupClass-method residuals,MultipleGroupClass-method 
+#' fitted,MultipleGroupClass-method 
 #' @param data a \code{matrix} or \code{data.frame} that consists of
 #' numerically ordered data, with missing data coded as \code{NA}
 #' @param model an object or named list of objects returned from \code{confmirt.model()} declaring how
@@ -31,47 +33,45 @@
 #' \item{\code{'intercepts'}}{to constrain all the intercepts to be equal across all groups, note for 
 #' nominal models this also includes the category specific slope parameters}}
 #' Additionally, specifying specific item name bundles (from \code{colnames(data)}) will 
-#' constrain all freely estimated parameters in each item to be equal accross groups. This is useful
+#' constrain all freely estimated parameters in each item to be equal across groups. This is useful
 #' for selecting 'anchor' items for vertical and horizontal scaling, and for detecting differential item 
-#' functioning (DIF) accross groups
+#' functioning (DIF) across groups
 #' @param guess initial (or fixed) values for the pseudo-guessing parameter. Can be 
 #' entered as a single value to assign a global guessing parameter or may be entered as
 #' a numeric vector for each item
 #' @param upper initial (or fixed) upper bound parameters for 4-PL model. Can be 
 #' entered as a single value to assign a global upper bound parameter or may be entered as a 
 #' numeric vector corresponding to each item
-#' @param SE logical; estimate the standard errors? Calls the MHRM subroutine for a stochastic approximation or 
-#' the Bock and Leiberman method (for EM only)
-#' @param SE.type type of estimation method to use for calculating the parameter information matrix. 
-#' Can be \code{'MHRM'} for stocastic estimation, or \code{'BL'} for the Bock and Leiberman approach (EM only). 
-#' Note that \code{'MHRM'} may be faster and more accurate than \code{'BL'} when there are 2 or more factors. 
-#' Bootstrapped standard errors are also possible but must be run with the \code{\link{boot.mirt}} function
+#' @param SE logical; estimate the information matrix for standard errors? 
+#' @param SE.type see \code{\link{mirt}} for more details
 #' @param D a numeric value used to adjust the logistic metric to be more similar to a normal
 #' cumulative density curve. Default is 1.702
-#' @param SEtol tollerance value used to stop the MHRM estimation when \code{SE = TRUE}. Lower values
-#' will take longer but may be more stable for computing the information matrix
 #' @param verbose logical; display iteration history during estimation?
 #' @param draws the number of Monte Carlo draws to estimate the log-likelihood
 #' @param quadpts the number of quadratures to be used per dimensions when \code{method = 'EM'}
 #' @param calcNull logical; calculate the Null model for fit statics (e.g., TLI)?
+#' @param cl a cluster object from the \code{parallel} package
 #' @param bfactor logical; use the dimensional reduction algorithm if the factor pattern is a bifactor
 #' model (has exactly 1 general factor and 1 specific factor for each item). Only applicable when 
 #' \code{method = 'EM'}
-#' @param prev.mod an optional input object of class \code{'MultipleGroupClass'} to quickly 
-#' change the starting values of the current estimation.
-#' If a parameter in the current model is being freely estimated then it's value will be set to whatever the 
-#' corresponding value was in this input object. This is useful when testing nested models since the starting 
-#' values should be much closer to the new ML values. Note that this method requires that all the \code{itemtype}
-#' values be identical between models
 #' @param method a character indicating whether to use the EM (\code{'EM'}) or the MH-RM 
 #' (\code{'MHRM'}) algorithm
+#' @param type type of plot to view; can be \code{'info'} to show the test
+#' information function, \code{'infocontour'} for the test information contours, 
+#' \code{'SE'} for the test standard error function, and \code{'RE'} for the relative efficiency plot
+#' @param theta_angle numeric values ranging from 0 to 90 used in \code{plot}
+#' @param npts number of quadrature points to be used for plotting features.
+#' Larger values make plots look smoother
+#' @param rot allows rotation of the 3D graphics
+#' @param x an object of class \code{mirt} to be plotted or printed
+#' @param y an unused variable to be ignored
+#' @param key see \code{\link{mirt}} for details
 #' @param itemtype see \code{\link{mirt}} for details
 #' @param constrain see \code{\link{mirt}} for details
 #' @param grsm.block see \code{\link{mirt}} for details
 #' @param rsm.block see \code{\link{mirt}} for details
 #' @param parprior see \code{\link{mirt}} for details
 #' @param pars see \code{\link{mirt}} for details
-#' @param debug logical; turn on debugging features?
 #' @param object an object of class \code{confmirtClass}
 #' @param object2 an object of class \code{confmirtClass}
 #' @param digits the number of significant digits to be rounded
@@ -81,11 +81,11 @@
 #' \describe{
 #' \item{NCYCLES}{max number of cycles; default 2000 for MHRM and 300 for EM}
 #' \item{MAXQUAD}{maximum number of quadratures; default 10000}
-#' \item{MSTEPMAXIT}{number of M-step iterations; default 15}
+#' \item{MSTEPTOL}{convergence threshold for Mstep; default is \code{TOL/100}}
 #' \item{BURNIN}{number of burn in cycles (stage 1); default 150}
 #' \item{SEMCYCLES}{number of SEM cycles (stage 2); default 50}
 #' \item{KDRAWS}{number of parallel MH sets to be drawn; default 1}
-#' \item{TOL}{minimum threshold tolerance for convergence of MH-RM, must occur on three consecutive
+#' \item{TOL}{minimum threshold tolerance for convergence. If MH-RM, must occur on three consecutive
 #' occations; default .001} 
 #'   \item{set.seed}{seed number used during estimation. Default is 12345}       
 #'   \item{gain}{a vector of three values specifying the numerator, exponent, and subtracted
@@ -95,20 +95,31 @@
 #' }
 #' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
 #' @seealso
-#' \code{\link{expand.table}}, \code{\link{key2binary}}, \code{\link{simdata}},
-#' \code{\link{confmirt.model}}, \code{\link{fscores}}, \code{\link{fitIndices}}
+#' \code{\link{expand.table}}, \code{\link{key2binary}}, \code{\link{confmirt.model}}, \code{\link{mirt}},
+#' \code{\link{confmirt}}, \code{\link{bfactor}}, \code{\link{multipleGroup}}, \code{\link{mixedmirt}}, 
+#' \code{\link{wald}}, \code{\link{itemplot}}, \code{\link{fscores}}, \code{\link{fitIndices}}, 
+#' \code{\link{extract.item}}, \code{\link{iteminfo}}, \code{\link{testinfo}}, \code{\link{probtrace}}, 
+#' \code{\link{boot.mirt}}, \code{\link{imputeMissing}}, \code{\link{itemfit}}, \code{\link{mod2values}},
+#' \code{\link{read.mirt}}, \code{\link{simdata}}, \code{\link{createItem}}
 #' @keywords models
 #' @usage 
-#' multipleGroup(data, model, group, itemtype = NULL, guess = 0, upper = 1, SE = FALSE, SE.type = 'MHRM',
-#' SEtol = .001,  invariance = '', pars = NULL, method = 'EM', constrain = NULL, 
+#' multipleGroup(data, model, group, itemtype = NULL, guess = 0, upper = 1, SE = FALSE, SE.type = 'SEM',
+#' invariance = '', pars = NULL, method = 'EM', constrain = NULL, 
 #' parprior = NULL, calcNull = TRUE, draws = 3000, quadpts = NULL, grsm.block = NULL, rsm.block = NULL, 
-#' prev.mod = NULL, bfactor = FALSE, D = 1.702, technical = list(), debug = FALSE, verbose = TRUE, ...)
+#' bfactor = FALSE, key = NULL, D = 1.702, cl = NULL, technical = list(), verbose = TRUE, ...)
 #' 
 #' \S4method{coef}{MultipleGroupClass}(object, digits = 3, verbose = TRUE, ...)
 #'
 #' \S4method{summary}{MultipleGroupClass}(object, digits = 3, verbose = TRUE, ...)
 #' 
 #' \S4method{anova}{MultipleGroupClass}(object, object2)
+#' 
+#' \S4method{residuals}{MultipleGroupClass}(object, ...)
+#'
+#' \S4method{fitted}{MultipleGroupClass}(object, ...)
+#' 
+#' \S4method{plot}{MultipleGroupClass}(x, y, type = 'info', npts = 50, theta_angle = 45, 
+#' rot = list(xaxis = -70, yaxis = 30, zaxis = 10), ...)
 #'
 #' @export multipleGroup
 #' @examples
@@ -128,20 +139,18 @@
 #' mod_configural <- multipleGroup(dat, models, group = group) #completely seperate analyses
 #' 
 #' # prev.mod can save precious iterations and help to avoid local minimums
-#' mod_metric <- multipleGroup(dat, models, group = group, invariance=c('slopes'), 
-#'                             prev.mod = mod_configural) #equal slopes
+#' mod_metric <- multipleGroup(dat, models, group = group, invariance=c('slopes')) #equal slopes
 #' mod_scalar2 <- multipleGroup(dat, models, group = group, #equal intercepts, free variance and means
-#'                              invariance=c('slopes', 'intercepts', 'free_varcov','free_means'),
-#'                              prev.mod = mod_configural)
+#'                              invariance=c('slopes', 'intercepts', 'free_varcov','free_means'))
 #' mod_scalar1 <- multipleGroup(dat, models, group = group,  #fixed means
-#'                              invariance=c('slopes', 'intercepts', 'free_varcov'),
-#'                              prev.mod = mod_configural)    
+#'                              invariance=c('slopes', 'intercepts', 'free_varcov'))    
 #' mod_fullconstrain <- multipleGroup(dat, models, group = group, 
-#'                              invariance=c('slopes', 'intercepts'),
-#'                              prev.mod = mod_configural)  
+#'                              invariance=c('slopes', 'intercepts'))  
 #'                              
 #' summary(mod_scalar2)
 #' coef(mod_scalar2)
+#' residuals(mod_scalar2)
+#' plot(mod_configural)
 #' itemplot(mod_configural, 2)  
 #' itemplot(mod_configural, 2, type = 'RE') 
 #' 
@@ -167,22 +176,21 @@
 #' #loop over items (in practice, run in parallel to increase speed)                             
 #' estmodels <- vector('list', ncol(dat))
 #' for(i in 1:ncol(dat))
-#'     estmodels[[i]] <- multipleGroup(dat, models, group = group, prev.mod=refmodel,
-#'                              invariance=c('free_means', 'free_varcov', itemnames[-i]),
-#'                              verbose = FALSE) 
+#'     estmodels[[i]] <- multipleGroup(dat, models, group = group, verbose = FALSE,
+#'                              invariance=c('free_means', 'free_varcov', itemnames[-i])) 
 #'                              
 #' (anovas <- lapply(estmodels, anova, object2=refmodel))
 #' 
 #' #family-wise error control
-#' p <- do.call(c, lapply(anovas, function(x) x[2,9]))
+#' p <- do.call(rbind, lapply(anovas, function(x) x[2,9]))
 #' p.adjust(p, method = 'BH') 
 #' 
 #' #same as above, except only test if slopes vary (1 df)
 #' estmodels <- vector('list', ncol(dat))
 #' for(i in 1:ncol(dat))
-#'     estmodels[[i]] <- multipleGroup(dat, models, group = group, prev.mod=refmodel,
-#'                              invariance=c('free_means', 'free_varcov', 'intercepts', itemnames[-i]),
-#'                              verbose = FALSE)  #constrain all intercepts
+#'     estmodels[[i]] <- multipleGroup(dat, models, group = group, verbose = FALSE, #constrain all intercepts
+#'                              invariance=c('free_means', 'free_varcov', 'intercepts', itemnames[-i]))
+#'                                
 #'                              
 #' (anovas <- lapply(estmodels, anova, object2=refmodel))
 #' 
@@ -219,12 +227,11 @@
 #' 
 #' #EM approach (not as accurate with 3 factors, but generally good for quick model comparisons)
 #' mod_configural <- multipleGroup(dat, models, group = group) #completely seperate analyses
-#' mod_metric <- multipleGroup(dat, models, group = group, invariance=c('slopes'), 
-#'      prev.mod=mod_configural) #equal slopes
-#' mod_scalar <- multipleGroup(dat, models, group = group, prev.mod=mod_configural, #equal means, slopes, intercepts
+#' mod_metric <- multipleGroup(dat, models, group = group, invariance=c('slopes')) #equal slopes
+#' mod_scalar <- multipleGroup(dat, models, group = group, #equal means, slopes, intercepts
 #'                              invariance=c('slopes', 'intercepts', 'free_varcov'))   
 #' mod_fullconstrain <- multipleGroup(dat, models, group = group, #equal means, slopes, intercepts
-#'                              invariance=c('slopes', 'intercepts'), prev.mod=mod_configural)
+#'                              invariance=c('slopes', 'intercepts'))
 #'                              
 #' anova(mod_metric, mod_configural)
 #' anova(mod_scalar, mod_metric)
@@ -241,26 +248,57 @@
 #' anova(mod_metric, mod_configural)
 #' anova(mod_scalar, mod_metric)
 #' anova(mod_fullconstrain, mod_scalar)
+#' 
+#' ############
+#' #polytomous item example
+#' set.seed(12345)
+#' a <- matrix(abs(rnorm(15,1,.3)), ncol=1)
+#' d <- matrix(rnorm(15,0,.7),ncol=1)    
+#' d <- cbind(d, d-1, d-2)
+#' itemtype <- rep('graded', nrow(a))
+#' N <- 1000    
+#' dataset1 <- simdata(a, d, N, itemtype)
+#' dataset2 <- simdata(a, d, N, itemtype, mu = .1, sigma = matrix(1.5))
+#' dat <- rbind(dataset1, dataset2)
+#' group <- c(rep('D1', N), rep('D2', N))    
+#' models <- confmirt.model('F1 = 1-15')
+#' models2 <- confmirt.model('
+#'    F1 = 1-10
+#'    F2 = 10-15')
+#' 
+#' mod_configural <- multipleGroup(dat, models, group = group) 
+#' plot(mod_configural)
+#' plot(mod_configural, type = 'SE')
+#' itemplot(mod_configural, 1)
+#' itemplot(mod_configural, 1, type = 'info')
+#' fs <- fscores(mod_configural)
+#' head(fs[["D1"]])
+#' fscores(mod_configural, method = 'EAPsum')
+#' 
+#' mod_configural2 <- multipleGroup(dat, models2, group = group) 
+#' plot(mod_configural2, type = 'infocontour')
+#' plot(mod_configural2, type = 'SE')
+#' plot(mod_configural2, type = 'RE')
+#' itemplot(mod_configural2, 10)
+#' 
 #' }
 multipleGroup <- function(data, model, group, itemtype = NULL, guess = 0, upper = 1, 
-                          SE = FALSE, SE.type = 'MHRM', SEtol = .001, invariance = '', pars = NULL,  
-                          method = 'EM',
-                          constrain = NULL, parprior = NULL, calcNull = TRUE, draws = 3000, 
-                          quadpts = NULL, grsm.block = NULL, rsm.block = NULL, prev.mod = NULL,
-                          bfactor = FALSE, D = 1.702, technical = list(), debug = FALSE, 
-                          verbose = TRUE, ...)
-{   
-    if(debug == 'Main') browser()
+                          SE = FALSE, SE.type = 'SEM', invariance = '', pars = NULL,  
+                          method = 'EM', constrain = NULL, parprior = NULL, calcNull = TRUE, 
+                          draws = 3000, quadpts = NULL, grsm.block = NULL, rsm.block = NULL, 
+                          bfactor = FALSE, key = NULL, D = 1.702, cl = NULL, 
+                          technical = list(), verbose = TRUE, ...)
+{       
     Call <- match.call()            
     invariance.check <- invariance %in% c('free_means', 'free_varcov')
     if(all(invariance.check) && length(constrain) == 0)
         stop('Model is not identified without further constrains (may require additional anchoring items).')
     mod <- ESTIMATION(data=data, model=model, group=group, invariance=invariance, 
-                      itemtype=itemtype, guess=guess, upper=upper, nested.mod=prev.mod, 
-                      pars=pars, constrain=constrain, SE=SE, SEtol=SEtol, grsm.block=grsm.block,
+                      itemtype=itemtype, guess=guess, upper=upper, 
+                      pars=pars, constrain=constrain, SE=SE, grsm.block=grsm.block,
                       parprior=parprior, quadpts=quadpts, method=method, D=D, rsm.block=rsm.block,
-                      technical = technical, debug = debug, verbose = verbose, calcNull=calcNull, 
-                      BFACTOR=bfactor, SE.type=SE.type, ...)
+                      technical = technical, verbose = verbose, calcNull=calcNull, 
+                      BFACTOR=bfactor, SE.type = SE.type, cl=cl, key=key, ...)
     if(is(mod, 'MultipleGroupClass'))
         mod@Call <- Call
     return(mod)    
