@@ -1,15 +1,20 @@
 context('mirtTwo')
 
 test_that('poly', {
-    modp1 <- mirt(Science, 1)
+    modp1 <- mirt(Science, 1, verbose=FALSE)
     expect_is(modp1, 'ConfirmatoryClass')          
-    modp2 <- mirt(Science, 2)
+    vals <- mirt(Science, 1, large = TRUE, verbose=FALSE)
+    modp1 <- mirt(Science, 1, large = vals, verbose=FALSE)
+    expect_is(modp1, 'ConfirmatoryClass')              
+    modp1 <- mirt(Science, 1, SE=TRUE, SE.type = 'SEM', verbose=FALSE)
+    expect_is(modp1, 'ConfirmatoryClass')          
+    modp2 <- mirt(Science, 2, verbose=FALSE)
     expect_is(modp2, 'ExploratoryClass')
-    modp3 <- mirt(Science, 1, constrain = list(c(1,5)), parprior = list(c(2,'norm',0,1)))
+    modp3 <- mirt(Science, 1, constrain = list(c(1,5)), parprior = list(c(2,'norm',0,1)), verbose=FALSE)
     expect_is(modp3, 'ConfirmatoryClass')
-    modp4 <- mirt(Science, 1, itemtype = c(rep('graded',3), 'nominal'))
+    modp4 <- mirt(Science, 1, itemtype = c(rep('graded',3), 'nominal'), verbose=FALSE)
     expect_is(modp4, 'ConfirmatoryClass')
-    modp5 <- mirt(Science, 1, itemtype = c(rep('graded',3), 'gpcm'), SE = TRUE)
+    modp5 <- mirt(Science, 1, itemtype = c(rep('graded',3), 'gpcm'), SE = TRUE, verbose=FALSE)
     expect_is(modp5, 'ConfirmatoryClass')
     
     fm1 <- fscores(modp1, verbose = FALSE)
@@ -38,10 +43,10 @@ test_that('poly', {
     d <- matrix(c(1,0.5,-.5,-1), 10, 4, byrow = TRUE)
     c <- seq(-1, 1, length.out=10)
     data <- simdata(a, d + c, 2000, itemtype = rep('graded',10))
-    sv <- mirt(data, 1, itemtype = 'grsm', pars = 'values')
+    sv <- mirt(data, 1, itemtype = 'grsm', pars = 'values', verbose=FALSE)
     sv[,5] <- c(as.vector(t(cbind(a,d,c))),0,1)    
-    grsm <- mirt(data, 1, itemtype = 'grsm', pars = sv, calcNull= FALSE)
-    rsm <- mirt(data, 1, itemtype = 'rsm', calcNull= FALSE)
+    grsm <- mirt(data, 1, itemtype = 'grsm', pars = sv, calcNull= FALSE, verbose=FALSE)
+    rsm <- mirt(data, 1, itemtype = 'rsm', calcNull= FALSE, verbose=FALSE)
     expect_is(grsm, 'ConfirmatoryClass')    
     expect_is(rsm, 'ConfirmatoryClass')    
     
@@ -51,6 +56,5 @@ test_that('poly', {
     iinfo <- iteminfo(x, Theta)
     expect_is(iinfo, 'matrix')    
     tinfo <- testinfo(modp1, Theta)
-    expect_is(tinfo, 'matrix')
-    
+    expect_is(tinfo, 'matrix')    
 })
\ No newline at end of file
